#include "Graupner_HOTT.h"

extern int alarm_state;
extern float V_0;
extern float V_1;
extern float V_2;
extern float V_Glob;
extern float I_0;
extern float Cap_mAh;

// Calculate CheckSum
uint8_t checksum (uint8_t *data)
{
	int i=0;
	uint8_t sum = 0;
	for (i = 0; i <= 43; i++)
	{
	sum = sum + data [i];
	}
	return sum;
}

/*
 Alarm
 */
void setAlarm(int Alarm){
	_hott_serial_buffer[2] = Alarm;
}

/*
 Hauptspannung: Werte +0.1 bis +53.5
 */
void setMainVoltage(float amp){
  int val = (int) (amp * 10);
  uint8_t low =  (uint8_t) (val&255);
  uint8_t high = (uint8_t)(val >> 8);
  _hott_serial_buffer[30] = low;
  _hott_serial_buffer[31] = high;
}
/*
 Akku Nummer 1 = Spannungssensor 1, Werte +0.1 bis +53.5
 */
void setVoltage1(float volt){
  int val = (int) (volt * 10);
  uint8_t low =  (uint8_t) (val&255);
  uint8_t high = (uint8_t)(val >> 8);
  _hott_serial_buffer[20] = low;
  _hott_serial_buffer[21] = high;
}
/*
 Akku Nummer 2 = Spannungssensor 2, Werte +0.1 bis +53.5
 */
void setVoltage2(float volt){
  int val = (int) (volt * 10);
  uint8_t low =  (uint8_t) (val&255);
  uint8_t high = (uint8_t)(val >> 8);
  _hott_serial_buffer[22] = low;
  _hott_serial_buffer[23] = high;
}
/*
 Einzelzellspannung:  +0.1 bis +5.08 Volt
 */
void setCell(int i, float zelle){
	_hott_serial_buffer[i+6] = (int) 100*zelle/2;
}
/*
 Temperatur 1
 */
void setTemp1(int Temp){
	_hott_serial_buffer[24] = (int) (Temp+0x14);
}
/*
 Temperatur 2
 */
void setTemp2(int Temp){
	_hott_serial_buffer[25] = (int) (Temp+0x14);
}
/*
 Ampere: Werte +0.1 bis +53.5
 */
void setAmpere(float amp){
  int val = (uint16_t) (amp * 10);
  uint8_t low =  (uint8_t) (val&255);
  //Alternativ: uint8_t high = (uint8_t) ((val&65280)&256);
  uint8_t high = (uint8_t)(val >> 8);
  _hott_serial_buffer[28] = low;
  _hott_serial_buffer[29] = high;
}
/*
 milliAmpere: Werte +10 bis +65535
 */
void setMilliAmpere(int mA){
  mA = mA;///10;
  uint8_t low =  (uint8_t) (mA&255);
  //Alternativ: uint8_t high = (uint8_t) ((val&65280)&256);
  uint8_t high = (uint8_t)(mA >> 8);
  _hott_serial_buffer[32] = low;
  _hott_serial_buffer[33] = high;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////

/*
 HÃƒÂ¶he in Metern (relativ): -500 bis +9999 [m], sinnvolle Werte z.B. 300
 */
void setAltitudeInMeters(uint16_t hoehe){
  uint8_t  hunderter = (int) (hoehe+500)/256;
  uint8_t  einer = hoehe-(hunderter*256-500);
  _hott_serial_buffer[24] = hunderter;
  _hott_serial_buffer[23] = einer;
}


//Init Hott Buffer
// Param: void
// Initialize HOTT output buffer
void init_HOTT_Buffer()
{
	int i=0;

	uint8_t _hott_buffer[] = {
            0x7C,
            HOTTV4_ELECTRICAL_AIR_MODULE,
            0x00, /* Alarm */
            HOTTV4_ELECTRICAL_AIR_SENSOR_ID,
            0x00, 0x00, /* Alarm Value 1 and 2 */
            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, /* Low Voltage Cell 1-7 in 2mV steps */
            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, /* High Voltage Cell 1-7 in 2mV steps */
            0x00, 0x00, /* Battetry 1 LSB/MSB in 100mv steps, 50 == 5V */
            0x00, 0x00, /* Battetry 2 LSB/MSB in 100mv steps, 50 == 5V */
            0x14, /* Temp 1, Offset of 20. 20 == 0C */
            0x14, /* Temp 2, Offset of 20. 20 == 0C */
            0xF4, 0x01, /* Height. Offset -500. 500 == 0 */
            0x00, 0x00, /* Current LSB, MSB 1 = 0.1A */
            0x00, 0x00, /* Drive Voltage */
            0x00, 0x00,  /* mAh */
            0x48, 0x00, /* m2s */
            0x78, /* m3s */
            0x00, 0x00, /* RPM. 10er steps, 300 == 3000rpm */
            0x00, /* Electric minutes */
            0x00, /* Electric seconds */
            0x00, /* Speed */
            0x00, /* Version Number */
            0x7D, /* End sign */
            0x00 /* Checksum */
          };

	for(i=0;i<44;i++)
	{
		_hott_serial_buffer[i] = _hott_buffer[i];
	}
}

//void Update HOTT Buffer
// Param: void
// Updates HOTT output buffer
void Update_HOTT_Buffer(void)
{


	setMainVoltage(V_Glob);
	setVoltage1(50.0f);
	setAmpere(I_0);
	setMilliAmpere(Cap_mAh/10);
	setCell(0,V_0);
	setCell(1,V_1);
	setCell(2,V_2);
	setTemp1(25.0f);
	setTemp2(46.1f);

	setAlarm(0x1f);

/*
	if(alarm_state >=20)
	{
		setAlarm(ALARM_LOW_VOLTAGE);
		setMainVoltage(9.92f);
	}
	else
	{
		setAlarm(ALARM_NONE);
		setMainVoltage(11.92f);
	}

	if(alarm_state >=40)
	{
		alarm_state=0;
	}
	*/

	//alarm_state++;

	_hott_serial_buffer[44] = checksum(_hott_serial_buffer);
}


//Simple delay
void delay(int j)
{
	int i;

	for(i=0;i<j;i++)
	{
	}

}

//Transmit HOTT Buffer
// Param: void
// Transmit HOTT output buffer
void Transmitt_HOTT_Buffer()
{
	int i;

	USART_ITConfig(USART1, USART_IT_RXNE, DISABLE);
	delay(0xafff);
	Update_HOTT_Buffer();

	for(i=0;i<45;i++)
	{
		USART_SendData(USART1, _hott_serial_buffer[i]);
		delay(0xfff);
	}
	USART_ITConfig(USART1, USART_IT_RXNE, ENABLE);
}
