#include "stm32f10x.h"
#include "stm32f10x_flash.h"
#include "stm32f10x_rcc.h"
#include "stm32f10x_adc.h"
#include "stm32f10x_gpio.h"
#include "stm32f10x_usart.h"
#include "stm32f10x_rtc.h"
#include "misc.h"

#include "Graupner_HOTT.h"

/** Define global variables */
#define RX_BUFER_SIZE 16
uint8_t RxData[RX_BUFER_SIZE];
int iRx;
int alarm_state=0;

float V_0_a = 0.0010900099f;
float V_1_a = 0.0023216374f;
float V_2_a = 0.0034609676f;

float I_0_a = 0.0309756098f;
unsigned int I_0_offset = 540;

float V_0;
float V_1;
float V_2;
float V_Glob;
float I_0=0;
float Cap_mAh=0;


//Initialize all GPIOs
void InitGpio(void);
void InitUSART(void);
void NVIC_Configuration(void);
//Init ADC
void ADC_Configuration(void);
//Get Samples from Chanel as Average
unsigned int Sample(int channel, unsigned int samples);

//Init RTC for secound counter
void Init_RTC(void);
//Start RTC
void Start_RTC(void);
//Stop RTC
void Stop_RTC(void);

//Get Sensor Data
void GetSensorData();

int main(void)
{
	int i=0;

	//HOTT Buffer
	init_HOTT_Buffer();
	//Init GPIO
	InitGpio();
	//ADC
	ADC_Configuration();
	//RTC
	Init_RTC();
	//Interupts
	NVIC_Configuration();
	//USART
	InitUSART();

	Start_RTC();

    while(1)
    {
    	if (iRx>=2)
		{
			//UART_Send(LPC_UART, RxData, iRx, NONE_BLOCKING);

			//Check if Binary request
			if(RxData[iRx-2] == HOTT_BINARY_MODE_REQUEST_ID)
			{
				//Request EAM?
				if(RxData[iRx-1] == HOTT_TELEMETRY_EAM_SENSOR_ID)
				{
					iRx = 0;
					Transmitt_HOTT_Buffer();
					//UART_Send(LPC_UART, (uint8_t *)"Transmitt\n", sizeof("Transmitt\n") - 1, NONE_BLOCKING);
				}
			}

			else
			{
				for(i=0;i<(iRx-1);i++)
				{
					RxData[i]=RxData[i+1];
				}
				iRx--;
			}


		}

    	//Get Sensor Data
    	GetSensorData();
    }
}

//Get Sensor Data
void GetSensorData()
{
	unsigned int adc_value0=0;
	unsigned int adc_value1=0;
	unsigned int adc_value2=0;
	unsigned int adc_value3=0;

	adc_value0 = Sample(0,8);
	adc_value1 = Sample(1,8);
	adc_value2 = Sample(2,8);
	adc_value3 = Sample(3,8);

	V_0 = adc_value0*V_0_a;
	V_1 = adc_value1*V_1_a;
	V_2 = adc_value2*V_2_a;

	//Set Pack Voltage
	V_Glob=V_2;

	//Calculate other Cell_Voltages
	V_2-=V_1;
	V_1-=V_0;

	I_0 = (adc_value3-I_0_offset)*I_0_a;

	if(I_0<0)
	{
		I_0=0.0f;
	}
}


//Initialize all GPIOs
void InitGpio()
{
    GPIO_InitTypeDef  GPIO_InitStructure;

    //Clock Peripherial
    RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOC, ENABLE);
    // Enable USART1 and GPIOA clock
    RCC_APB2PeriphClockCmd(RCC_APB2Periph_USART1 | RCC_APB2Periph_GPIOA, ENABLE);
    RCC_APB2PeriphClockCmd( RCC_APB2Periph_AFIO, ENABLE);

    // Configure the GPIO_LED pin
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_9;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_Init(GPIOC, &GPIO_InitStructure);

    /* Configure USART1 Tx (PA.09) as alternate function push-pull */
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_9;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_Init(GPIOA, &GPIO_InitStructure);

    /* Configure USART1 Rx (PA.10) as input floating */
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_10;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
    GPIO_Init(GPIOA, &GPIO_InitStructure);

    //ADC 1
	GPIO_InitStructure.GPIO_Mode  = GPIO_Mode_AIN;
	GPIO_InitStructure.GPIO_Pin   = GPIO_Pin_0 |GPIO_Pin_1 | GPIO_Pin_2| GPIO_Pin_3;		// that's ADC1 (PA1 on STM32)
	GPIO_Init(GPIOA, &GPIO_InitStructure);
}

void InitUSART(void)
{
	USART_InitTypeDef USART_InitStructure;

	USART_InitStructure.USART_BaudRate = 19200;
	USART_InitStructure.USART_WordLength = USART_WordLength_8b;
	USART_InitStructure.USART_StopBits = USART_StopBits_1;
	USART_InitStructure.USART_Parity = USART_Parity_No;
	USART_InitStructure.USART_HardwareFlowControl = USART_HardwareFlowControl_None;
	USART_InitStructure.USART_Mode = USART_Mode_Rx | USART_Mode_Tx;

	USART_Init(USART1, &USART_InitStructure);

	/* Enable USART1 */
	USART_Cmd(USART1, ENABLE);

	/* Enable the USART1 Receive interrupt: this interrupt is generated when the
	         USART1 receive data register is not empty */
	USART_ITConfig(USART1, USART_IT_RXNE, ENABLE);
}

//Init ADC ----------------------------------------------------------------------------------
void ADC_Configuration(void)
{
	ADC_InitTypeDef ADC_InitStructure;

	//clock for ADC (max 14MHz)
	RCC_ADCCLKConfig (RCC_PCLK2_Div2);
	// enable ADC system clock
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_ADC1, ENABLE);

	// define ADC config
	ADC_InitStructure.ADC_Mode = ADC_Mode_Independent;
	ADC_InitStructure.ADC_ScanConvMode = DISABLE;
	ADC_InitStructure.ADC_ContinuousConvMode = DISABLE;	// we work in continuous sampling mode
	ADC_InitStructure.ADC_ExternalTrigConv = ADC_ExternalTrigConv_None;
	ADC_InitStructure.ADC_DataAlign = ADC_DataAlign_Right;
	ADC_InitStructure.ADC_NbrOfChannel = 1;

	ADC_RegularChannelConfig(ADC1,ADC_Channel_0, 1,ADC_SampleTime_28Cycles5); // define regular conversion config
	ADC_RegularChannelConfig(ADC1,ADC_Channel_1, 1,ADC_SampleTime_28Cycles5); // define regular conversion config
	ADC_RegularChannelConfig(ADC1,ADC_Channel_2, 1,ADC_SampleTime_28Cycles5); // define regular conversion config
	ADC_RegularChannelConfig(ADC1,ADC_Channel_3, 1,ADC_SampleTime_28Cycles5); // define regular conversion config
	ADC_Init ( ADC1, &ADC_InitStructure);	//set config of ADC1

	// enable ADC
	ADC_Cmd (ADC1,ENABLE);	//enable ADC1

	//	ADC calibration (optional, but recommended at power on)
	ADC_ResetCalibration(ADC1);	// Reset previous calibration
	while(ADC_GetResetCalibrationStatus(ADC1));
	ADC_StartCalibration(ADC1);	// Start new calibration (ADC must be off at that time)
	while(ADC_GetCalibrationStatus(ADC1));

	// start conversion
	//ADC_Cmd (ADC1,ENABLE);	//enable ADC1
	//ADC_SoftwareStartConvCmd(ADC1, ENABLE);

	// debug information
	//RCC_ClocksTypeDef forTestOnly;
	//RCC_GetClocksFreq(&forTestOnly);
}

//Get Samples from Chanel as Average ----------------------------------------------------------------------------------
unsigned int Sample(int channel, unsigned int samples)
{
	unsigned int values=0;
	unsigned int i=0;

	//Select Channel
	ADC_RegularChannelConfig(ADC1,channel, 1,ADC_SampleTime_28Cycles5); // define regular conversion config

	for(i=0;i<samples;i++)
	{
		ADC_SoftwareStartConvCmd(ADC1, ENABLE);					// start ONE conversion
		while(ADC_GetFlagStatus(ADC1, ADC_FLAG_EOC) == RESET);	// wait end of conversion
		values += ADC_GetConversionValue(ADC1);
	}

	return (values/samples);
}

/**
  * @brief  Configures the nested vectored interrupt controller.
  * @param  None
  * @retval None
  */
void NVIC_Configuration(void)
{
  NVIC_InitTypeDef NVIC_InitStructure;

  /* Enable the USARTx Interrupt */
  NVIC_InitStructure.NVIC_IRQChannel = USART1_IRQn;
  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  NVIC_Init(&NVIC_InitStructure);

  /* Configure one bit for preemption priority */
  NVIC_PriorityGroupConfig(NVIC_PriorityGroup_1);

  /* Enable the RTC Interrupt */
  NVIC_InitStructure.NVIC_IRQChannel = RTC_IRQn;
  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 1;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  NVIC_Init(&NVIC_InitStructure);
}


//Start RTC
void Start_RTC(void)
{
	/* Enable RTC Clock */
	RCC_RTCCLKCmd(ENABLE);
}
//Stop RTC
void Stop_RTC(void)
{
	/* Enable RTC Clock */
	RCC_RTCCLKCmd(DISABLE);
}

//Init RTC for secound counter
void Init_RTC(void)
{
	/* Enable PWR and BKP clocks */
	    RCC_APB1PeriphClockCmd(RCC_APB1Periph_PWR | RCC_APB1Periph_BKP, ENABLE);

	    /* Allow access to BKP Domain */
	    PWR_BackupAccessCmd(ENABLE);

	    /* Reset Backup Domain */
	    //BKP_DeInit();

	    /* Enable LSE */
	    RCC_LSEConfig(RCC_LSE_ON);
	    /* Wait till LSE is ready */
	    while (RCC_GetFlagStatus(RCC_FLAG_LSERDY) == RESET)
	    {}

	    /* Select LSE as RTC Clock Source */
	    RCC_RTCCLKConfig(RCC_RTCCLKSource_LSE);

	    /* Enable RTC Clock */
	    RCC_RTCCLKCmd(ENABLE);

	    /* Wait for RTC registers synchronization */
	    RTC_WaitForSynchro();

	    /* Wait until last write operation on RTC registers has finished */
	    RTC_WaitForLastTask();

	    /* Enable the RTC Second */
	    RTC_ITConfig(RTC_IT_SEC, ENABLE);

	    /* Wait until last write operation on RTC registers has finished */
	    RTC_WaitForLastTask();

	    /* Set RTC prescaler: set RTC period to 1sec */
	    RTC_SetPrescaler(32767); /* RTC period = RTCCLK/RTC_PR = (32.768 KHz)/(32767+1) */

	    /* Wait until last write operation on RTC registers has finished */
	    RTC_WaitForLastTask();

	    /* Change the current time */
		RTC_SetCounter(0);
		/* Wait until last write operation on RTC registers has finished */
		RTC_WaitForLastTask();

}

/******************************************************************************/
/*            STM32F10x Peripherals Interrupt Handlers                        */
/******************************************************************************/

/**
  * @brief  This function handles USARTx global interrupt request.
  * @param  None
  * @retval None
  */
void USART1_IRQHandler(void)
{
	unsigned char i=0;

    if ((USART1->SR & USART_FLAG_RXNE) != (u16)RESET)
	{
			i = USART_ReceiveData(USART1);
			if(iRx == RX_BUFER_SIZE)
			{
				RxData[iRx] = i;
				iRx = 0;
			}
			else
			{
				RxData[iRx++] = i;
			}

	}
}

/**
  * @brief  This function handles RTC global interrupt request.
  * @param  None
  * @retval None
  */
void RTC_IRQHandler(void)
{
    if (RTC_GetITStatus(RTC_IT_SEC) != RESET)
    {
        /* Clear the RTC Second interrupt */
        RTC_ClearITPendingBit(RTC_IT_SEC);


        Cap_mAh+=((I_0*1000.0f)/3600.0f);


        /* Wait until last write operation on RTC registers has finished */
        RTC_WaitForLastTask();
    }
}
